use cgmath::{Vector2, Vector3, Zero};
use utilities::{impl_reprc, prelude::*};
use vulkan_rs::prelude::*;

impl_reprc!(
    #[derive(Copy, Debug)]
    pub struct PositionNormalColorUVBone {
        #[assume_reprc]
        pub position: Vector3<f32>,

        #[assume_reprc]
        filler_p: f32,

        #[assume_reprc]
        pub normal: Vector3<f32>,

        #[assume_reprc]
        filler_n: f32,

        #[assume_reprc]
        pub uv: Vector2<f32>,

        #[assume_reprc]
        filler_u: [f32; 2],

        #[assume_reprc]
        pub joints: [i32; 4],
        #[assume_reprc]
        pub weights: [f32; 4],
    }
);

impl PositionNormalColorUVBone {
    pub fn new(
        position: Vector3<f32>,
        normal: Vector3<f32>,
        uv: Vector2<f32>,
        joints: [i32; 4],
        weights: [f32; 4],
    ) -> Self {
        Self {
            position,
            filler_p: 1.0,
            normal,
            filler_n: 0.0,
            uv,
            filler_u: [0.0, 0.0],
            joints,
            weights,
        }
    }
}

impl Default for PositionNormalColorUVBone {
    fn default() -> Self {
        PositionNormalColorUVBone {
            position: Vector3::zero(),
            filler_p: 1.0,
            normal: Vector3::zero(),
            filler_n: 0.0,
            uv: Vector2::zero(),
            filler_u: [0.0, 0.0],
            joints: [0; 4],
            weights: [0.0; 4],
        }
    }
}

impl VertexInputDescription for PositionNormalColorUVBone {
    fn attributes() -> Vec<VkVertexInputAttributeDescription> {
        vec![
            // position
            VkVertexInputAttributeDescription {
                location: 0,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 0,
            },
            // normal
            VkVertexInputAttributeDescription {
                location: 1,
                binding: 0,
                format: VK_FORMAT_R32G32B32_SFLOAT,
                offset: 16,
            },
            // uvs
            VkVertexInputAttributeDescription {
                location: 3,
                binding: 0,
                format: VK_FORMAT_R32G32_SFLOAT,
                offset: 32,
            },
            // joints
            VkVertexInputAttributeDescription {
                location: 4,
                binding: 0,
                format: VK_FORMAT_R32G32B32A32_SINT,
                offset: 48,
            },
            // weights
            VkVertexInputAttributeDescription {
                location: 5,
                binding: 0,
                format: VK_FORMAT_R32G32B32A32_SFLOAT,
                offset: 64,
            },
        ]
    }
}
