use std::path::Path;

use anyhow::Result;

use ecs::*;
use engine::prelude::{cgmath::vec3, *};
use skybox::SkyBox;

fn main() -> Result<()> {
    let mut world_builder = World::builder();

    Engine::new::<GameState>(EngineCreateInfo::default(), &mut world_builder)?;

    world_builder.add_system(GameState::update);
    world_builder.resources.insert(GameState::Startup);

    // let dir = Path::new("C:/Users/M.Huebner/Downloads/Space Skybox Generator/Export");
    let dir = Path::new("/home/michaelh/Sync/skybox");
    SkyBox::new(
        &mut world_builder,
        [
            dir.join("left.png"),
            dir.join("right.png"),
            dir.join("front.png"),
            dir.join("back.png"),
            dir.join("top.png"),
            dir.join("bottom.png"),
        ]
        .into_iter(),
    )?;

    let view = world_builder.resources.get_mut::<Scene>().view_mut();
    let camera = view.camera_mut();

    camera.set_eye_offset(vec3(0.0, 1.0, 0.0));
    view.update_buffer()?;

    world_builder.build().run()
}

enum GameState {
    Startup,
    Loading,
    Menu,
    Game(Game),
}

impl GameState {
    fn update(world: &mut World) -> Result<bool> {
        match world.resources.get_mut_unchecked::<Self>() {
            GameState::Startup => (),
            GameState::Loading => (),
            GameState::Menu => (),
            GameState::Game(game) => game.update(world)?,
        }

        Ok(true)
    }
}

impl EventConsumer for GameState {
    fn event(&mut self, world: &mut World, event: EngineEvent<'_>) -> Result<()> {
        match self {
            GameState::Startup => (),
            GameState::Loading => (),
            GameState::Menu => (),
            GameState::Game(game) => game.event(world, event)?,
        }

        Ok(())
    }
}

struct Game {}

impl Game {
    fn update(&mut self, world: &mut World) -> Result<()> {
        Ok(())
    }

    fn event(&mut self, world: &mut World, event: EngineEvent<'_>) -> Result<()> {
        Ok(())
    }
}
